@@ -427,6 +427,10 @@ main(int argc, char **argv)
 	parser_init(langmap, gtags_parser);
 	if (vflag && file_list)
 		fprintf(stderr, " Using '%s' as a file list.\n", file_list);
+	/*
+	 * Start statistics.
+	 */
+	init_statistics();
 	/*
 	 * incremental update.
 	 */
@@ -445,9 +449,9 @@ main(int argc, char **argv)
 		if (!test("f", makepath(dbpath, dbname(GPATH), NULL)))
 			die("Old version tag file found. Please remake it.");
 		(void)incremental(dbpath, cwd);
+		print_statistics(statistics);
 		exit(0);
 	}
-	init_statistics();
 	/*
 	 * create GTAGS and GRTAGS
 	 */
@@ -463,6 +467,7 @@ main(int argc, char **argv)
 		strbuf_puts(sb, "mkid");
 		if (vflag)
 			strbuf_puts(sb, " -v");
+		strbuf_sprintf(sb, " --file='%s/ID'", dbpath);
 		if (vflag) {
 #ifdef __DJGPP__
 			if (is_unixy())	/* test for 4DOS as well? */
@@ -497,6 +502,7 @@ main(int argc, char **argv)
 int
 incremental(const char *dbpath, const char *root)
 {
+	STATISTICS_TIME *tim;
 	struct stat statp;
 	time_t gtags_mtime;
 	STRBUF *addlist = strbuf_open(0);
@@ -507,6 +513,7 @@ incremental(const char *dbpath, const char *root)
 	const char *path;
 	unsigned int id, limit;
 
+	tim = statistics_time_start("Time of inspecting %s and %s.", dbname(GTAGS), dbname(GRTAGS));
 	if (vflag) {
 		fprintf(stderr, " Tag found in '%s'.\n", dbpath);
 		fprintf(stderr, " Incremental updating.\n");
@@ -626,44 +633,48 @@ normal_update:
 		}
 	}
 	gpath_close();
+	statistics_time_end(tim);
 	/*
 	 * execute updating.
 	 */
-	if (!idset_empty(deleteset) || strbuf_getlen(addlist) > 0) {
-		updatetags(dbpath, root, deleteset, addlist);
-		updated = 1;
-	}
-	if (strbuf_getlen(deletelist) + strbuf_getlen(addlist_other) > 0) {
-		const char *start, *end, *p;
-
-		if (vflag)
-			fprintf(stderr, "[%s] Updating '%s'.\n", now(), dbname(GPATH));
-		gpath_open(dbpath, 2);
-		if (strbuf_getlen(deletelist) > 0) {
-			start = strbuf_value(deletelist);
-			end = start + strbuf_getlen(deletelist);
-
-			for (p = start; p < end; p += strlen(p) + 1)
-				gpath_delete(p);
-		}
-		if (strbuf_getlen(addlist_other) > 0) {
-			start = strbuf_value(addlist_other);
-			end = start + strbuf_getlen(addlist_other);
-
-			for (p = start; p < end; p += strlen(p) + 1)
-				gpath_put(p, GPATH_OTHER);
-		}
-		gpath_close();
-		updated = 1;
-	}
-	if (updated) {
+	if ((!idset_empty(deleteset) || strbuf_getlen(addlist) > 0) ||
+	    (strbuf_getlen(deletelist) + strbuf_getlen(addlist_other) > 0))
+	{
 		int db;
+
+		updated = 1;
+		tim = statistics_time_start("Time of updating %s and %s.", dbname(GTAGS), dbname(GRTAGS));
+		if (!idset_empty(deleteset) || strbuf_getlen(addlist) > 0)
+			updatetags(dbpath, root, deleteset, addlist);
+		if (strbuf_getlen(deletelist) + strbuf_getlen(addlist_other) > 0) {
+			const char *start, *end, *p;
+
+			if (vflag)
+				fprintf(stderr, "[%s] Updating '%s'.\n", now(), dbname(GPATH));
+			gpath_open(dbpath, 2);
+			if (strbuf_getlen(deletelist) > 0) {
+				start = strbuf_value(deletelist);
+				end = start + strbuf_getlen(deletelist);
+
+				for (p = start; p < end; p += strlen(p) + 1)
+					gpath_delete(p);
+			}
+			if (strbuf_getlen(addlist_other) > 0) {
+				start = strbuf_value(addlist_other);
+				end = start + strbuf_getlen(addlist_other);
+
+				for (p = start; p < end; p += strlen(p) + 1)
+					gpath_put(p, GPATH_OTHER);
+			}
+			gpath_close();
+		}
 		/*
 		 * Update modification time of tag files
 		 * because they may have no definitions.
 		 */
 		for (db = GTAGS; db < GTAGLIM; db++)
 			utime(makepath(dbpath, dbname(db), NULL), NULL);
+		statistics_time_end(tim);
 	}
 	if (vflag) {
 		if (updated)
