@@ -1,22 +1,21 @@
 /*
- * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003
+ * Copyright (c) 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2010
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
  *
- * GNU GLOBAL is free software; you can redistribute it and/or modify
+ * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2, or (at your option)
- * any later version.
- *
- * GNU GLOBAL is distributed in the hope that it will be useful,
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
+ * 
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -41,48 +40,6 @@ static const char *dirs[]    = {NULL, DEFS,         REFS,        SYMS};
 static const char *kinds[]   = {NULL, "definition", "reference", "symbol"};
 static const char *options[] = {NULL, "",           "r",         "s"};
 
-static char command[MAXFILLEN];
-
-/*
- * Open duplicate object index file.
- *
- *	i)	db	tag type(GTAGS, GRTAGS, GSYMS)
- *	r)	op	file pointer
- */
-static FILE *
-open_dup_file(int db, int count)
-{
-	char path[MAXPATHLEN];
-	FILE *op;
-
-	snprintf(path, sizeof(path), "%s/%s/%d.%s", distpath, dirs[db], count, HTML);
-	if (cflag) {
-		snprintf(command, sizeof(command), "gzip -c >%s", path);
-		op = popen(command, "w");
-		if (op == NULL)
-			die("cannot execute command '%s'.", command);
-	} else {
-		op = fopen(path, "w");
-		if (op == NULL)
-			die("cannot create file '%s'.", path);
-	}
-	return op;
-}
-/*
- * Close duplicate object index file.
- *
- *	i)	op	file pointer
- */
-static void
-close_dup_file(FILE *op)
-{
-	if (cflag) {
-		if (pclose(op) != 0)
-			die("'%s' failed.", command);
-	} else {
-		fclose(op);
-	}
-}
 /*
  * Make duplicate object index.
  *
@@ -94,11 +51,12 @@ int
 makedupindex(void)
 {
 	STRBUF *sb = strbuf_open(0);
+	STRBUF *tmp = strbuf_open(0);
 	STRBUF *command = strbuf_open(0);
 	int definition_count = 0;
 	char srcdir[MAXPATHLEN];
 	int db;
-	char buf[1024];
+	FILEOP *fileop = NULL;
 	FILE *op = NULL;
 	FILE *ip = NULL;
 
@@ -109,7 +67,8 @@ makedupindex(void)
 		int writing = 0;
 		int count = 0;
 		int entry_count = 0;
-		char *ctags_x, tag[IDENTLEN], prev[IDENTLEN], first_line[MAXBUFLEN];
+		const char *ctags_xid, *ctags_x;
+		char tag[IDENTLEN], prev[IDENTLEN], first_line[MAXBUFLEN];
 
 		if (gtags_exist[db] == 0)
 			continue;
@@ -119,7 +78,7 @@ makedupindex(void)
 		 * construct command line.
 		 */
 		strbuf_reset(command);
-		strbuf_sprintf(command, "%s -x%s --nofilter=path", global_path, option);
+		strbuf_sprintf(command, "%s -x%s --result=ctags-xid --encode-path=\" \t\" --nofilter=path", global_path, option);
 		/*
 		 * Optimization when the --dynamic option is specified.
 		 */
@@ -131,16 +90,12 @@ makedupindex(void)
 		strbuf_puts(command, " \".*\"");
 		if ((ip = popen(strbuf_value(command), "r")) == NULL)
 			die("cannot execute command '%s'.", strbuf_value(command));
-		while ((ctags_x = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
-			SPLIT ptable;
-
-			if (split(ctags_x, 2, &ptable) < 2) {
-				recover(&ptable);
-				die("too small number of parts.(1)\n'%s'", ctags_x);
-			}
-			strlimcpy(tag, ptable.part[PART_TAG].start, sizeof(tag));
-			recover(&ptable);
+		while ((ctags_xid = strbuf_fgets(sb, ip, STRBUF_NOCRLF)) != NULL) {
+			char fid[MAXFIDLEN];
 
+			ctags_x = parse_xid(ctags_xid, fid, NULL);
+			/* tag name */
+			(void)strcpy_withterm(tag, ctags_x, sizeof(tag), ' ');
 			if (strcmp(prev, tag)) {
 				count++;
 				if (vflag)
@@ -150,50 +105,63 @@ makedupindex(void)
 						fputs_nl(gen_list_end(), op);
 						fputs_nl(body_end, op);
 						fputs_nl(gen_page_end(), op);
-						close_dup_file(op);
+						close_file(fileop);
 						html_count++;
 					}
 					writing = 0;
 					/*
-					 * cache record: " <file id> <entry number>"
+					 * cache record: " <fid>\0<entry number>\0"
 					 */
-					snprintf(buf, sizeof(buf), " %d %d", count - 1, entry_count);
-					cache_put(db, prev, buf);
+					strbuf_reset(tmp);
+					strbuf_putc(tmp, ' ');
+					strbuf_putn(tmp, count - 1);
+					strbuf_putc(tmp, '\0');
+					strbuf_putn(tmp, entry_count);
+					cache_put(db, prev, strbuf_value(tmp), strbuf_getlen(tmp) + 1);
 				}				
 				/* single entry */
 				if (first_line[0]) {
-					if (split(first_line, 4, &ptable) < 4) {
-						recover(&ptable);
-						die("too small number of parts.(2)\n'%s'", ctags_x);
-					}
-					snprintf(buf, sizeof(buf), "%s %s", ptable.part[PART_LNO].start, ptable.part[PART_PATH].start);
-					cache_put(db, prev, buf);
-					recover(&ptable);
+					char fid[MAXFIDLEN];
+					const char *ctags_x = parse_xid(first_line, fid, NULL);
+					const char *lno = nextelement(ctags_x);
+
+					strbuf_reset(tmp);
+					strbuf_puts_withterm(tmp, lno, ' ');
+					strbuf_putc(tmp, '\0');
+					strbuf_puts(tmp, fid);
+					cache_put(db, prev, strbuf_value(tmp), strbuf_getlen(tmp) + 1);
 				}
 				/*
 				 * Chop the tail of the line. It is not important.
 				 * strlimcpy(first_line, ctags_x, sizeof(first_line));
 				 */
-				strncpy(first_line, ctags_x, sizeof(first_line));
+				strncpy(first_line, ctags_xid, sizeof(first_line));
 				first_line[sizeof(first_line) - 1] = '\0';
 				strlimcpy(prev, tag, sizeof(prev));
 				entry_count = 0;
 			} else {
 				/* duplicate entry */
 				if (first_line[0]) {
+					char fid[MAXFIDLEN];
+					const char *ctags_x = parse_xid(first_line, fid, NULL);
+
 					if (!dynamic) {
-						op = open_dup_file(db, count);
+						char path[MAXPATHLEN];
+
+						snprintf(path, sizeof(path), "%s/%s/%d.%s", distpath, dirs[db], count, HTML);
+						fileop = open_output_file(path, cflag);
+						op = get_descripter(fileop);
 						fputs_nl(gen_page_begin(tag, SUBDIR), op);
 						fputs_nl(body_begin, op);
 						fputs_nl(gen_list_begin(), op);
-						fputs_nl(gen_list_body(srcdir, first_line), op);
+						fputs_nl(gen_list_body(srcdir, ctags_x, fid), op);
 					}
 					writing = 1;
 					entry_count++;
 					first_line[0] = 0;
 				}
 				if (!dynamic) {
-					fputs_nl(gen_list_body(srcdir, ctags_x), op);
+					fputs_nl(gen_list_body(srcdir, ctags_x, fid), op);
 				}
 				entry_count++;
 			}
@@ -207,28 +175,33 @@ makedupindex(void)
 				fputs_nl(gen_list_end(), op);
 				fputs_nl(body_end, op);
 				fputs_nl(gen_page_end(), op);
-				close_dup_file(op);
+				close_file(fileop);
 				html_count++;
 			}
 			/*
-			 * cache record: " <file id> <entry number>"
+			 * cache record: " <fid>\0<entry number>\0"
 			 */
-			snprintf(buf, sizeof(buf), " %d %d", count, entry_count);
-			cache_put(db, prev, buf);
+			strbuf_reset(tmp);
+			strbuf_putc(tmp, ' ');
+			strbuf_putn(tmp, count);
+			strbuf_putc(tmp, '\0');
+			strbuf_putn(tmp, entry_count);
+			cache_put(db, prev, strbuf_value(tmp), strbuf_getlen(tmp) + 1);
 		}
 		if (first_line[0]) {
-			SPLIT ptable;
+			char fid[MAXFIDLEN];
+			const char *ctags_x = parse_xid(first_line, fid, NULL);
+			const char *lno = nextelement(ctags_x);
 
-			if (split(first_line, 4, &ptable) < 4) {
-				recover(&ptable);
-				die("too small number of parts.(3)\n'%s'", ctags_x);
-			}
-			snprintf(buf, sizeof(buf), "%s %s", ptable.part[PART_LNO].start, ptable.part[PART_PATH].start);
-			cache_put(db, prev, buf);
-			recover(&ptable);
+			strbuf_reset(tmp);
+			strbuf_puts_withterm(tmp, lno, ' ');
+			strbuf_putc(tmp, '\0');
+			strbuf_puts(tmp, fid);
+			cache_put(db, prev, strbuf_value(tmp), strbuf_getlen(tmp) + 1);
 		}
 	}
 	strbuf_close(sb);
+	strbuf_close(tmp);
 	strbuf_close(command);
 	return definition_count;
 }
