@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1998, 1999, 2000, 2001, 2002, 2005
+ * Copyright (c) 1998, 1999, 2000, 2001, 2002, 2005, 2010
  *	Tama Communications Corporation
  *
  * This file is part of GNU GLOBAL.
@@ -223,6 +223,8 @@ configpath(void)
 		strbuf_puts(sb, DEBIANCONF);
 	else if (test("r", OLD_DEBIANCONF))
 		strbuf_puts(sb, OLD_DEBIANCONF);
+	else if (test("r", makepath(SYSCONFDIR, "gtags.conf", NULL)))
+		strbuf_puts(sb, makepath(SYSCONFDIR, "gtags.conf", NULL));
 	else
 		return NULL;
 	return strbuf_value(sb);
@@ -407,6 +409,9 @@ getconfs(const char *name, STRBUF *sb)
 		} else if (!strcmp(name, "localstatedir")) {
 			strbuf_puts(sb, LOCALSTATEDIR);
 			exist = 1;
+		} else if (!strcmp(name, "sysconfdir")) {
+			strbuf_puts(sb, SYSCONFDIR);
+			exist = 1;
 		}
 	}
 	return exist;
